/*
Copyright (c) 2013, Linear Technology Corp.(LTC)
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:

1. Redistributions of source code must retain the above copyright notice, this
   list of conditions and the following disclaimer.
2. Redistributions in binary form must reproduce the above copyright notice,
   this list of conditions and the following disclaimer in the documentation
   and/or other materials provided with the distribution.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR
ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

The views and conclusions contained in the software and documentation are those
of the authors and should not be interpreted as representing official policies,
either expressed or implied, of Linear Technology Corp.
*/
//`timescale 1ns / 1ps
//module 'serial_io' generates patterns for SPI signals, reads serial LTC2380_24 data and converts it to parallel using Altera MAXII
//GMH
//August 19, 2014  RevA

module serial_io(master_clk, 
								mosi, 
								miso, 
								data_latch, 
								sck, 
								conv, 
								aux0, 
								dout, 
								mcd,
								SEL_24_40L,
								sneaker_port_count,
								WRIN_I2C,
								ONEWORD_TWOWORDL,
								SLOW,
								mcd2,
								clk_gate,
								bus,
								reset_pulse,
								data_latch_gate);				// added 03/10/15
								
   input master_clk;	   									// external clock in
   input mcd;    												// mcd is a delayed version of master_clk
	input mcd2;													// mcd2 is a delayed version of master_clk to gate data_latch
	input SEL_24_40L;											// selects 24bit or 40bit format 1=24, 0=40
	input miso;													// sdo to cpld (serial data from ADC)
	input [15:0] sneaker_port_count;						// 16-bit word from PScope, stes divide ratio
	input	WRIN_I2C;											// WRIN_I2C=0 write to sneaker_port, WRIN_I2C=1 read ADC
	input ONEWORD_TWOWORDL;									// 1=1word 0=2words 2words required for >1.6Msps
	input SLOW;													// 0=SLOW
	input	reset_pulse;
   output mosi;			  									// sdi to adc (some parts may not use this line)
   output data_latch;										// latches data into cpld   (active high)
   output sck;													// gated version of external clock in
   output conv;												// convert or CS to ADC
   output aux0;  												// aux output   
   output 	[39:0] dout;									// 40-bit deserialized data from ADC
	output clk_gate;
	output [6:0] bus;
	output data_latch_gate;									// added 03/10/15
	//wire clk_count;
	
   reg 		[47:0] aux0_shift;
   reg 		[47:0] conv_shift;
	reg		[47:0] data_latch_gate_shift; //added 03/10/15
   reg 		[47:0] data_latch_shift;
   reg 		[47:0] delay_shift;												
   reg 		[47:0] mosi_shift;
   reg 		[47:0] sck_shift;
   reg 		[39:0] dout;	  					
   reg 		[7:0] delay;
   reg 		[6:0] clk_counter;
	assign 	bus[1] = reset_pulse;
	assign   bus[2] = sneaker_port_count[0];
	assign   bus[3] = sneaker_port_count[1];
   wire 		aux0;		 
   wire 		data_latch;				// added 03/10/15
	wire		data_latch_gate; //added 03/10/15
   wire 		master_clk;
   wire 		mcd;
	wire		mcd2;
	wire		SEL_24_40L;
	wire		ONEWORD_TWOWORDL;
	wire		SLOW;
   wire 		miso;
   wire 		mosi;
   wire 		sck;
	wire		[15:0] sneaker_port_count;
	wire		WRIN_I2C;
	wire		reset_pulse;
	reg 		[23:0] miso_def;
	reg 		[47:0] mosi_def;
	reg		[47:0] latch_gate;	//added 03/10/15
	reg 		[47:0] data_latch_def;
	reg 		[47:0] sck_def;
	reg 		[47:0] conv_def;
	reg 		[47:0] aux0_def;
	reg 		[47:0] delay_def;
	reg 		gate;
	
	wire		[7:0] delay_time;
   wire		[6:0] max_clk_cnt;
	
	reg gate_dly_1;																		//11/05/14
	reg gate_dly_2;																		//11/05/14
	reg gate_dly_3;																		//11/05/14
	reg clk_gate;
	reg slow_gate;																			//02/12/15
	
																	// SLOW=0 1 bit/conv (Slow Mode) SLOW and ONEWORD_TWOWORDL cannot both be low
																	// SEL_24_40L can only be low if ONEWORD_TWOWORDL=1
																	// ONEWORD_TWOWORDL=0 2 12bit words (Fast mode)
																	// ONEWORD_TWOWORDL=1 1 24bit or 40bit word (Normal)
	always @( SEL_24_40L or ONEWORD_TWOWORDL or SLOW)begin//
	
		if (SEL_24_40L==0)begin///							//Normal 40bit mode Divider =83
	
																	//   counts per delay	max clk count 		# of channels 	# of bits 
																	//   b16-b23(max 255)	b9-b15(max 127)	b5-b8(max 16)	b0-b4 (max 32)
																	//				36							48			    1		  		   40	   
          miso_def <= 24'H235E3F;    					//  0010 0011			0101 	111			0 001 	   1 1111   	                               												      
          mosi_def <= 48'H000000000000;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 
			latch_gate<= 48'HF80000000000;				//	 1111 1000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 //added 03/10/15
    data_latch_def <= 48'H500000000000;    			//  0101 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000	   	           	                               
           sck_def <= 48'H03FFFFFFFFFC;    			//  0000 0011 1111 1111 1111 1111 1111 1111 1111 1111 1111 1100    	    	                               
          conv_def <= 48'H07FFFFFFFFFC;    			//  0000 0111 1111 1111 1111 1111 1111 1111 1111 1111 1111 1100 	   	    	          	                           
          aux0_def <= 48'H000000000001;				//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
         delay_def <= 48'H000000000002;      		//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0010 
	
		end ///

	
		else begin///											// Normal 24bit mode			Divider = 66
				if (ONEWORD_TWOWORDL==1)begin////		
																	//   counts per delay	max clk count 		# of channels 	# of bits 
																	//   b16-b23(max 255)	b9-b15(max 127)	b5-b8(max 16)	b0-b4 (max 32)	
																	//				36							31			    1		  		   24	   
				miso_def 		<= 24'H233C3F;    					//  0010 0011			0011 	110			0 001 	   1 1111   	                               												      
				mosi_def 		<= 48'H000000000000;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 
					latch_gate  <= 48'H000070000000;				   //	 0000 0000 0000 0000 0111 0000 0000 0000 0000 0000 0000 0000	//added 03/10/15
				data_latch_def <= 48'H000020000000;    			//  0000 0000 0000 0000 0010 0000 0000 0000 0000 0000 0000 0000	  
				sck_def 			<= 48'H000007FFFFF8;    			//  0000 0000 0000 0000 0000 0111 1111 1111 1111 1111 1111 1000    	    	                               
				conv_def 		<= 48'H00000FFFFFFC;    			//  0000 0000 0000 0000 0000 1111 1111 1111 1111 1111 1111 1100 	   	    	          	                           
				aux0_def 		<= 48'H000000000001;					//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
				delay_def 		<= 48'H000000000002;      			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0010 	
				end////
				else begin////
																	// Fast 24bit mode Divider = 53
																	//   counts per delay	max clk count 		# of channels 	# of bits 
																	//   b16-b23(max 255)	b9-b15(max 127)	b5-b8(max 16)	b0-b4 (max 32)
																	//				12							20			    1		  		   24	   
				miso_def 		<= 24'H0B2638;    					//  0000 1011			0010 	011			0 001 	   1 1000   	                               												      
				mosi_def 		<= 48'H000000000000;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 
					latch_gate	<= 48'H00000000001C;					//	 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001 1100	//added 03/10/15
				data_latch_def <= 48'H000000000008;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 1000	
				sck_def 			<= 48'H00000007FF82;    			//  0000 0000 0000 0000 0000 0000 0000 0111 1111 1111 1000 0010    	    	                               
				conv_def 		<= 48'H0000000FFF80;    			//  0000 0000 0000 0000 0000 0000 0000 1111 1111 1111 1000 0000 	   	    	          	                           
				aux0_def 		<= 48'H000000000001;					//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
				delay_def 		<= 48'H000000000062;      			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0110 0010 
				end////
		end ///
	
		if (SLOW==0)begin///									// SLOW mode Divider = 52
																	//   counts per delay	max clk count 		# of channels 	# of bits 
																	//   b16-b23(max 255)	b9-b15(max 127)	b5-b8(max 16)	b0-b4 (max 32)
																	//				32							21			    1		  		   24	   
          miso_def <= 24'H1F2838;    					//  0001 1111			0010 	100			0 001 	   1 1000   	                               												      
          mosi_def <= 48'H000000000000;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000
			latch_gate<= 48'H00000000001C;				//	 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001 1100	//added 03/10/15
    data_latch_def <= 48'H000000000008;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 1000	 
           sck_def <= 48'H000000000002;    			//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0010    	    	                               
          conv_def <= 48'H0000001FFFE0;    			//  0000 0000 0000 0000 0000 0000 0001 1111 1111 1111 1110 0000 	   	    	          	                           
          aux0_def <= 48'H000000000001;				//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0001
         delay_def <= 48'H000000000020;      		//  0000 0000 0000 0000 0000 0000 0000 0000 0000 0000 0010 0000 
		end ///
	
	end	//
	
	assign max_clk_cnt =  miso_def[15:9];											// maximum count before clk_counter is reset	(max 127)     
	assign delay_time =  miso_def[23:16];											// number of master_clks to wait if delay goes high (max 255)
	assign data_latch = data_latch_shift[0] & gate_dly_3 ;					//	11/5/14 was data_latch_shift[0] & gate &!sck	
	assign data_latch_gate = data_latch_gate_shift[0];										//added 03/10/15
	assign mosi = mosi_shift[0];						
	assign aux0 = aux0_shift[0];
	assign sck = (sck_shift[0]& mcd) & (clk_gate | slow_gate) ;					
	wire 	 conv;
	assign conv = !(conv_shift[0])& WRIN_I2C;										//03/09/15 added & WRIN_I2C
	//assign clk_count= data_latch_shift[0];
	
   initial begin //
   delay <= 0;															
   clk_counter <= 7'b0000000;															// initialize counter loop   
   aux0_shift<= 48'H000000000000;
   conv_shift<= 48'H000000000000;
	latch_gate<= 48'H000000000000;													//added 03/10/15
   data_latch_shift<= 48'H000000000000;
   delay_shift<= 48'H000000000000;
   mosi_shift<= 48'H000000000000;
   sck_shift<= 48'H000000000000;
   dout<= 40'b0000000000000000000000000000000000000000;		    							      
   end //
																				
	dff dff_gate_dly_1(.d(0),.clk(mcd), .clrn(WRIN_I2C), .prn(!gate), .q(gate_dly_1));						//11/03/14	stretches gate signal long enough for 2 data_latch pulses
	dff dff_gate_dly_2(.d(gate_dly_1),.clk(mcd), .clrn(WRIN_I2C), .prn(!gate), .q(gate_dly_2));			//11/03/14
	dff dff_gate_dly_3(.d(gate_dly_2),.clk(mcd), .clrn(WRIN_I2C), .prn(!gate), .q(gate_dly_3));			//11/03/14
	
															//added 02/04/15
	always @ (sneaker_port_count or WRIN_I2C or SLOW)begin	//						//03/04/15 was always @ (sneaker_port_count or WRIN_I2C)			//clk_gate=0 sck disabled, clk_gate=1 sck enabled
	
		if (sneaker_port_count==1)begin	///									//sck is enabled when sneaker_port_counter=1
		clk_gate<=1;
		end///
		else begin///
		clk_gate<=0;
		end	///		
		
		if (sneaker_port_count <=24 & SLOW==0)begin	///					//02/12/15
		slow_gate<=1;
		end///
		else begin///
		slow_gate<=0;
		end///

		if (WRIN_I2C==0)begin///
		clk_gate<=0;																	//02/06/15  added to make sure clk_gate=0 when WRIN_I2C goes low
		slow_gate<=0;
		end///
	end//

	always @(posedge master_clk or  negedge WRIN_I2C)						// everything happens on pos edge of master clk or neg edge of WRIN_I2C

   if (WRIN_I2C==0)begin//
	clk_counter <= max_clk_cnt;			  										//if WRIN_I2C=0 reset clk_counter and strings
		data_latch_gate_shift <= latch_gate;									//added 03/10/15
	data_latch_shift <= data_latch_def;						
	conv_shift <= conv_def;
	mosi_shift <= mosi_def;
	sck_shift <= sck_def;
	aux0_shift <= aux0_def;
	delay_shift <= delay_def;
	gate <= 1'b0;			//added 02/06/15
	delay <=0;
   end//

else begin/// if master_clk=1
 
      if 	(delay_shift[0]==1)begin	//// 										
	     if (delay<delay_time)begin	/////									
		  delay <= delay +1'b1; 											
	     end	/////															
			else begin/////		if delay >=delay_time											
				if(delay >= delay_time) begin//////							
				clk_counter <= clk_counter-1'b1;	
					data_latch_gate_shift <= data_latch_gate_shift >> 1;					//added 03/10/15
				data_latch_shift <= data_latch_shift >> 1;				
				conv_shift <= conv_shift >> 1 ;							
				mosi_shift <= mosi_shift >> 1;							
				sck_shift <= sck_shift >> 1;							
				aux0_shift <= aux0_shift >> 1;							
				delay_shift <= delay_shift >> 1;						
				delay <= 0;		
					//	data_latch <= data_latch_shift[0] & gate_dly_3 /*& mcd2 */;		//030215 was & mcd2
				end //////													
			end /////															
		end////																
		else begin////			if delay_shift=0											
			if (clk_counter > 0) begin /////		  							
			clk_counter <= clk_counter - 1'b1;	
				data_latch_gate_shift <= data_latch_gate_shift >> 1;					//added 03/10/15
			data_latch_shift <= data_latch_shift >> 1;
			conv_shift <= conv_shift >> 1 ;
			mosi_shift <= mosi_shift >> 1;
			sck_shift <= sck_shift >> 1;
			aux0_shift <= aux0_shift >> 1;
			delay_shift <= delay_shift >>1;	
					//	data_latch <= data_latch_shift[0] & gate_dly_3 /*& mcd2 */;		//030215 was & mcd2
			if (sneaker_port_count==0) gate<=1'b1;									//gate signal is used to hold off data_latch until specified number of conversions is completed
			else gate<=1'b0;																//gate signal is stretched by gate_dly_3
			
			end /////	
			else begin ///// if clk_counter=0
		   clk_counter <= max_clk_cnt;			  									//if clk_counter=0 reset clk_counter 
				data_latch_gate_shift <= latch_gate;											//added 03/10/15
			data_latch_shift <= data_latch_def;						
			conv_shift <= conv_def;
			mosi_shift <= mosi_def;
			sck_shift <= sck_def;
			aux0_shift <= aux0_def;
			delay_shift <= delay_def;		
					//	data_latch <= data_latch_shift[0] & gate_dly_3 /*& mcd2 */;		//030215 was & mcd2
			end /////
		end////           
	end///															
always @(posedge sck) begin //														// sck is not inverted from master clk
			dout <= {dout[38:0],miso};													// latch data bit, place in proper location of dout
end //

endmodule